@@ -7,6 +7,7 @@ from .replace_policy import replace_policies
 from ..constants import INFERENCE_GENERIC_MODE, INFERENCE_SPECIALIZED_MODE
 from ..runtime.weight_quantizer import WeightQuantization
 from torch import nn
+from deepspeed import comm as dist
 
 
 class LinearAllreduce(nn.Module):
@@ -19,7 +20,7 @@ class LinearAllreduce(nn.Module):
     def forward(self, input):
         output = torch.matmul(input, self.weight)
         if self.mp_group is not None:
-            torch.distributed.all_reduce(output, group=self.mp_group)
+            dist.all_reduce(output, group=self.mp_group)
         if self.bias is not None:
             output += self.bias
         return output
@@ -41,7 +42,7 @@ class LinearLayer(nn.Module):
 class ReplaceWithTensorSlicing:
     def __init__(self, mp_group=None):
         if mp_group is not None:
-            self.gpu_index = torch.distributed.get_rank(group=mp_group)
+            self.gpu_index = dist.get_rank(group=mp_group)
         else:
             self.gpu_index = 0
 
@@ -247,7 +248,7 @@ def replace_transformer_layer(orig_layer_impl,
 
         if inference:
             if moe:
-                ep_world_size = torch.distributed.get_world_size()
+                ep_world_size = dist.get_world_size()
                 local_ep_size = 1 if num_experts < ep_world_size else num_experts // ep_world_size
 
                 transformer_config = transformer_inference.DeepSpeedMoEInferenceConfig(
@@ -351,8 +352,11 @@ def replace_transformer_layer(orig_layer_impl,
             # linear layer is created with [input, output] shape
             # transpose it here to reduce inference cost!
             def transpose(data):
+                # temp move to cpu to avoid requiring extra GPU memory during the reshape
+                data = data.to('cpu')
                 data.reshape(-1).copy_(data.transpose(-1, -2).contiguous().reshape(-1))
                 data = data.reshape(data.shape[-1], data.shape[-2])
+                data.to(torch.cuda.current_device())
                 return data
 
             if attn_linear_layer:
@@ -416,7 +420,7 @@ def replace_transformer_layer(orig_layer_impl,
 
             mpl_block = new_module.mlp
             if moe:
-                gpu_index = torch.distributed.get_rank()
+                gpu_index = dist.get_rank()
                 gpu_index = 0
                 for ep_index in range(local_ep_size):
                     mpl_block[ep_index].inter_w.data = _h4h_w[
@@ -460,7 +464,7 @@ def replace_transformer_layer(orig_layer_impl,
             new_module.norm_b.data = input_nb.to(torch.cuda.current_device())
         else:
             transformer_config = deepspeed.DeepSpeedTransformerConfig(
-                batch_size=micro_batch_size,
+                batch_size=micro_batch_size if micro_batch_size > 0 else 1,
                 hidden_size=config.hidden_size,
                 heads=config.num_attention_heads,
                 attn_dropout_ratio=config.attention_probs_dropout_prob,
