@@ -10,7 +10,7 @@ from torch.autograd import Function
 import time
 from ... import op_builder
 import torch.nn as nn
-import torch.distributed as dist
+from deepspeed import comm as dist
 # Cuda modules will be imported if needed
 inference_cuda_module = None
 
@@ -269,7 +269,10 @@ class DeepSpeedSelfAttentionFunction(Function):
                 linear_func = inference_cuda_module.linear_layer_fp16 if config.fp16 else \
                                     inference_cuda_module.linear_layer_fp32
 
-                qkv_out = linear_func(input, attn_qkvw, attn_qkvb)
+                qkv_out = linear_func(input,
+                                      attn_qkvw,
+                                      attn_qkvb,
+                                      DeepSpeedTransformerInference.layer_id)
             else:
                 qkv_func = inference_cuda_module.qkv_gemm_fp16 if config.fp16 else \
                                     inference_cuda_module.qkv_gemm_fp32
@@ -468,7 +471,7 @@ class DeepSpeedMLPFunction(Function):
                                          config.pre_layer_norm,
                                          False)
             else:
-                intermediate = mlp_gemm_func(input,
+                intermediate, residual_add = mlp_gemm_func(input,
                                              residual,
                                              bias,
                                              inter_w,
@@ -479,14 +482,16 @@ class DeepSpeedMLPFunction(Function):
                                              config.pre_layer_norm,
                                              config.mlp_after_attn)
                 output = vector_matmul_func(intermediate, output_w, False)
-        inference_cuda_module.residual_add(output,
-                                           residual,
-                                           input,
-                                           output_b,
-                                           bias if bias is not None else output_b,
-                                           config.mp_size,
-                                           config.mlp_after_attn,
-                                           bias is not None)
+        inference_cuda_module.residual_add(
+            output,
+            residual if config.pre_layer_norm else residual_add,
+            input,
+            output_b,
+            bias if bias is not None else output_b,
+            config.mp_size,
+            config.mlp_after_attn,
+            bias is not None,
+            config.pre_layer_norm)
         if mp_group is not None and dist.get_world_size(group=mp_group) > 1:
             dist.all_reduce(output, group=mp_group)
         return output
@@ -668,7 +673,7 @@ class DeepSpeedTransformerInference(nn.Module):
                                       self.config.epsilon)
 
             output = output.to(input_type)
-        #print(f'[{torch.distributed.get_rank()}] {self.config.layer_id}: {output.norm()}')
+        #print(f'[{deepspeed.comm.get_rank()}] {self.config.layer_id}: {output.norm()}')
         #exit()
         if get_present:
             output = (output, presents)
