@@ -95,12 +95,14 @@ class FlopsProfiler(object):
             module.__post_hook_handle__ = module.register_forward_hook(post_hook)
 
             def start_time_hook(module, input):
+                torch.cuda.synchronize()
                 module.__start_time__ = time.time()
 
             module.__start_time_hook_handle__ = module.register_forward_pre_hook(
                 start_time_hook)
 
             def end_time_hook(module, input, output):
+                torch.cuda.synchronize()
                 module.__duration__ += time.time() - module.__start_time__
 
             module.__end_time_hook_handle__ = module.register_forward_hook(end_time_hook)
@@ -146,8 +148,7 @@ class FlopsProfiler(object):
         def add_or_reset_attrs(module):
             module.__flops__ = 0
             module.__macs__ = 0
-            module.__params__ = sum(p.numel() for p in module.parameters()
-                                    if p.requires_grad)
+            module.__params__ = sum(p.numel() for p in module.parameters())
             module.__start_time__ = 0
             module.__duration__ = 0
 
@@ -285,7 +286,7 @@ class FlopsProfiler(object):
         print('{:<60}  {:<8}'.format(
             'params of model = params per GPU * mp_size: ',
             params_to_string(total_params *
-                             (self.ds_engine.mp_world_size) if self.ds_engine else 1)))
+                             ((self.ds_engine.mp_world_size) if self.ds_engine else 1))))
 
         print('{:<60}  {:<8}'.format('fwd MACs per GPU: ', macs_to_string(total_macs)))
 
@@ -294,7 +295,7 @@ class FlopsProfiler(object):
         print('{:<60}  {:<8}'.format(
             'fwd flops of model = fwd flops per GPU * mp_size: ',
             num_to_string(total_flops *
-                          (self.ds_engine.mp_world_size) if self.ds_engine else 1)))
+                          ((self.ds_engine.mp_world_size) if self.ds_engine else 1))))
 
         fwd_latency = self.get_total_duration()
         if self.ds_engine and self.ds_engine.wall_clock_breakdown():
@@ -337,7 +338,7 @@ class FlopsProfiler(object):
             macs = get_module_macs(module)
             items = [
                 params_to_string(params),
-                "{:.2%} Params".format(params / total_params),
+                "{:.2%} Params".format(params / total_params if total_params else 0),
                 macs_to_string(macs),
                 "{:.2%} MACs".format(0.0 if total_macs == 0 else macs / total_macs),
             ]
@@ -532,7 +533,7 @@ def _conv_flops_compute(input,
     batch_size = input.shape[0]
     in_channels = input.shape[1]
     out_channels = weight.shape[0]
-    kernel_dims = list(weight.shape[-2:])
+    kernel_dims = list(weight.shape[2:])
     input_dims = list(input.shape[2:])
 
     length = len(input_dims)
@@ -573,7 +574,7 @@ def _conv_trans_flops_compute(
     batch_size = input.shape[0]
     in_channels = input.shape[1]
     out_channels = weight.shape[0]
-    kernel_dims = list(weight.shape[-2:])
+    kernel_dims = list(weight.shape[2:])
     input_dims = list(input.shape[2:])
 
     length = len(input_dims)
@@ -732,7 +733,6 @@ def _einsum_flops_compute(equation, *operands):
     np_arrs = [np.zeros(s) for s in input_shapes]
     optim = np.einsum_path(equation, *np_arrs, optimize="optimal")[1]
     for line in optim.split("\n"):
-        print(line.lower())
         if "optimized flop" in line.lower():
             flop = int(float(line.split(":")[-1]))
             return flop, 0
@@ -747,7 +747,15 @@ def _tensor_addmm_flops_compute(self, mat1, mat2, *, beta=1, alpha=1, out=None):
     return 2 * macs + _prod(self.shape), macs
 
 
-def _elementwise_flops_compute(input, other, *, out=None):
+def _mul_flops_compute(input, other, *, out=None):
+    return _elementwise_flops_compute(input, other)
+
+
+def _add_flops_compute(input, other, *, alpha=1, out=None):
+    return _elementwise_flops_compute(input, other)
+
+
+def _elementwise_flops_compute(input, other):
     if not torch.is_tensor(input):
         if torch.is_tensor(other):
             return _prod(other.shape), 0
@@ -856,11 +864,11 @@ def _patch_tensor_methods():
     torch.addmm = wrapFunc(torch.addmm, _addmm_flops_compute)
     torch.Tensor.addmm = wrapFunc(torch.Tensor.addmm, _tensor_addmm_flops_compute)
 
-    torch.mul = wrapFunc(torch.mul, _elementwise_flops_compute)
-    torch.Tensor.mul = wrapFunc(torch.Tensor.mul, _elementwise_flops_compute)
+    torch.mul = wrapFunc(torch.mul, _mul_flops_compute)
+    torch.Tensor.mul = wrapFunc(torch.Tensor.mul, _mul_flops_compute)
 
-    torch.add = wrapFunc(torch.add, _elementwise_flops_compute)
-    torch.Tensor.add = wrapFunc(torch.Tensor.add, _elementwise_flops_compute)
+    torch.add = wrapFunc(torch.add, _add_flops_compute)
+    torch.Tensor.add = wrapFunc(torch.Tensor.add, _add_flops_compute)
 
     torch.einsum = wrapFunc(torch.einsum, _einsum_flops_compute)
 
@@ -1125,8 +1133,9 @@ def get_module_duration(module):
 
 def get_model_profile(
     model,
-    input_res,
-    input_constructor=None,
+    input_shape=None,
+    args=[],
+    kwargs={},
     print_profile=True,
     detailed=True,
     module_depth=-1,
@@ -1144,12 +1153,13 @@ def get_model_profile(
 
         model = torchvision.models.alexnet()
         batch_size = 256
-        flops, macs, params = get_model_profile(model=model, input_res= (batch_size, 3, 224, 224)))
+        flops, macs, params = get_model_profile(model=model, input_shape=(batch_size, 3, 224, 224)))
 
     Args:
         model ([torch.nn.Module]): the PyTorch model to be profiled.
-        input_res (list): input shape or input to the input_constructor
-        input_constructor (func, optional): input constructor. If specified, the constructor is applied to input_res and the constructor output is used as the input to the model. Defaults to None.
+        input_shape (tuple): input shape to the model. If specified, the model takes a tensor with this shape as the only positional argument.
+        args (list): list of positional arguments to the model.
+        kwargs (dict): dictionary of keyword arguments to the model.
         print_profile (bool, optional): whether to print the model profile. Defaults to True.
         detailed (bool, optional): whether to print the detailed model profile. Defaults to True.
         module_depth (int, optional): the depth into the nested modules. Defaults to -1 (the inner most modules).
@@ -1162,43 +1172,33 @@ def get_model_profile(
     Returns:
         The number of floating-point operations, multiply-accumulate operations (MACs), and parameters in the model.
     """
-    assert type(input_res) is tuple
-    assert len(input_res) >= 1
-    assert isinstance(model, nn.Module)
+    assert isinstance(model, nn.Module), "model must be a PyTorch module"
     prof = FlopsProfiler(model)
     model.eval()
-    for _ in range(warm_up):
-        if input_constructor:
-            input = input_constructor(input_res)
-            _ = model(**input)
-        else:
-            try:
-                batch = torch.ones(()).new_empty(
-                    (*input_res,
-                     ),
-                    dtype=next(model.parameters()).dtype,
-                    device=next(model.parameters()).device,
-                )
-            except StopIteration:
-                batch = torch.ones(()).new_empty((*input_res, ))
-            _ = model(batch)
 
-    prof.start_profile(ignore_list=ignore_modules)
-
-    if input_constructor:
-        input = input_constructor(input_res)
-        _ = model(**input)
-    else:
+    if input_shape is not None:
+        assert type(input_shape) is tuple, "input_shape must be a tuple"
+        assert len(input_shape) >= 1, "input_shape must have at least one element"
         try:
-            batch = torch.ones(()).new_empty(
-                (*input_res,
+            input = torch.ones(()).new_empty(
+                (*input_shape,
                  ),
                 dtype=next(model.parameters()).dtype,
                 device=next(model.parameters()).device,
             )
         except StopIteration:
-            batch = torch.ones(()).new_empty((*input_res, ))
-        _ = model(batch)
+            input = torch.ones(()).new_empty((*input_shape, ))
+
+        args = [input]
+
+    assert (len(args) > 0) or (len(kwargs) > 0), "args and/or kwargs must be specified if input_shape is None"
+
+    for _ in range(warm_up):
+        _ = model(*args, **kwargs)
+
+    prof.start_profile(ignore_list=ignore_modules)
+
+    _ = model(*args, **kwargs)
 
     flops = prof.get_total_flops()
     macs = prof.get_total_macs()
