@@ -1,4 +1,5 @@
 from .logging import logger, log_dist
-from .distributed import init_distributed
+#from .distributed import init_distributed
 from .groups import *
+from .nvtx import instrument_w_nvtx
 from deepspeed.runtime.dataloader import RepeatingLoader
