@@ -10,9 +10,9 @@ from .transformer import TransformerBuilder
 from .stochastic_transformer import StochasticTransformerBuilder
 from .utils import UtilsBuilder
 from .async_io import AsyncIOBuilder
-from .builder import get_default_compute_capabilities
 from .transformer_inference import InferenceBuilder
 from .quantizer import QuantizerBuilder
+from .builder import get_default_compute_capabilities, OpBuilder
 
 # TODO: infer this list instead of hard coded
 # List of all available ops
@@ -25,8 +25,8 @@ __op_builders__ = [
     TransformerBuilder(),
     StochasticTransformerBuilder(),
     AsyncIOBuilder(),
-    InferenceBuilder(),
     UtilsBuilder(),
-    QuantizerBuilder()
+    QuantizerBuilder(),
+    InferenceBuilder()
 ]
 ALL_OPS = {op.name: op for op in __op_builders__}
